@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 2468 2007-06-12 00:18:34Z wwalker $"
-__version__   = "$Revision: 2468 $"
-__date__      = "$Date: 2007-06-11 20:18:34 -0400 (Mon, 11 Jun 2007) $"
+__id__        = "$Id: orca.py 3032 2007-10-12 19:31:00Z richb $"
+__version__   = "$Revision: 3032 $"
+__date__      = "$Date: 2007-10-12 15:31:00 -0400 (Fri, 12 Oct 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -45,10 +45,11 @@ except:
     pass
 
 import getopt
+import gnome
 import os
 import signal
-import string
 import time
+import unicodedata
 
 import atspi
 import braille
@@ -442,7 +443,8 @@ def _isPrintableKey(event_string):
     else:
         unicodeString = event_string.decode("UTF-8")
         reply = (len(unicodeString) == 1) \
-                and (unicodeString.isalnum() or unicodeString.isspace())
+                and (unicodeString.isalnum() or unicodeString.isspace()
+                or unicodedata.category(unicodeString)[0] in ('P', 'S'))
     debug.println(debug.LEVEL_FINEST,
                   "orca._echoPrintableKey: returning: %s" % reply)
     return reply
@@ -523,6 +525,30 @@ def _isActionKey(event_string):
                   "orca._echoActionKey: returning: %s" % reply)
     return reply
 
+class KeyEventType:
+    """Definition of available key event types."""
+
+    """An alphanumeric or punctuation key event."""
+    PRINTABLE = 'PRINTABLE'
+
+    """A modifier key event."""
+    MODIFIER = 'MODIFIER'
+
+    """A locking key event."""
+    LOCKING = 'LOCKING'
+
+    """A locking key lock event."""
+    LOCKING_LOCKED = 'LOCKING_LOCKED'
+
+    """A locking key unlock event."""
+    LOCKING_UNLOCKED = 'LOCKING_UNLOCKED'
+
+    """A function key event."""
+    FUNCTION = 'FUNCTION'
+
+    """An action key event."""
+    ACTION = 'ACTION'
+
 def _keyEcho(event):
     """If the keyEcho setting is enabled, check to see what type of key
     event it is and echo it via speech, if the user wants that type of
@@ -546,11 +572,6 @@ def _keyEcho(event):
     debug.println(debug.LEVEL_FINEST,
                   "orca._keyEcho: string to echo: %s" % event_string)
 
-    voices = settings.voices
-    voice = voices[settings.DEFAULT_VOICE]
-
-    lockState = None
-
     # If key echo is enabled, then check to see what type of key event
     # it is and echo it via speech, if the user wants that type of key
     # echoed.
@@ -560,24 +581,25 @@ def _keyEcho(event):
         if _isPrintableKey(event_string):
             if not settings.enablePrintableKeys:
                 return
-
-            if event_string.isupper():
-                voice = voices[settings.UPPERCASE_VOICE]
+            type = KeyEventType.PRINTABLE
 
         elif _isModifierKey(event_string):
             if not settings.enableModifierKeys:
                 return
+            type = KeyEventType.MODIFIER
 
         elif _isLockingKey(event_string):
             if not settings.enableLockingKeys:
                 return
+            type = KeyEventType.LOCKING
 
             modifiers = event.modifiers
+
             if event_string == "Caps_Lock":
                 if modifiers & (1 << atspi.Accessibility.MODIFIER_SHIFTLOCK):
-                    lockState = " " + _("off")
+                    type = KeyEventType.LOCKING_UNLOCKED
                 else:
-                    lockState = " " + _("on")
+                    type = KeyEventType.LOCKING_LOCKED
 
             elif event_string == "Num_Lock":
                 # [[[TODO: richb - we are not getting a correct modifier
@@ -586,32 +608,26 @@ def _keyEcho(event):
                 # until this can be fixed.]]]
                 #
                 #if modifiers & (1 << atspi.Accessibility.MODIFIER_NUMLOCK):
-                #    event_string += " " + _("off")
+                #    type = KeyEventType.LOCKING_UNLOCKED
                 #else:
-                #    event_string += " " + _("on")
+                #    type = KeyEventType.LOCKING_LOCKED
                 pass
 
         elif _isFunctionKey(event_string):
             if not settings.enableFunctionKeys:
                 return
+            type = KeyEventType.FUNCTION
 
         elif _isActionKey(event_string):
             if not settings.enableActionKeys:
                 return
+            type = KeyEventType.ACTION
 
         else:
             debug.println(debug.LEVEL_FINEST,
                   "orca._keyEcho: event string not handled: %s" % event_string)
             return
 
-        # Check to see if there are localized words to be spoken for
-        # this key event.
-        #
-        event_string = keynames.getKeyName(event_string)
-
-        if lockState:
-            event_string += lockState
-
         debug.println(debug.LEVEL_FINEST,
                       "orca._keyEcho: speaking: %s" % event_string)
 
@@ -620,7 +636,7 @@ def _keyEcho(event):
         #
         orca_state.lastKeyEchoTime = time.time()
 
-        speech.speak(event_string, voice)
+        speech.speakKeyEvent(event_string, type)
 
 def _processKeyCaptured(event):
     """Called when a new key event arrives and orca_state.capturingKeys=True.
@@ -735,7 +751,6 @@ def _processKeyboardEvent(event):
         keyboardEvent.modifiers = keyboardEvent.modifiers \
                                   | (1 << settings.MODIFIER_ORCA)
 
-
     # Orca gets first stab at the event.  Then, the presenter gets
     # a shot. [[[TODO: WDW - might want to let the presenter try first?
     # The main reason this is staying as is is that we may not want
@@ -874,11 +889,15 @@ def loadUserSettings(script=None, inputEvent=None):
             reloaded = True
         except ImportError:
             debug.printException(debug.LEVEL_FINEST)
+        except:
+            debug.printException(debug.LEVEL_SEVERE)
     else:
         try:
             _userSettings = __import__("user-settings")
         except ImportError:
             debug.printException(debug.LEVEL_FINEST)
+        except:
+            debug.printException(debug.LEVEL_SEVERE)
 
     # If any settings were added to the command line, they take
     # precedence over everything else.
@@ -1039,6 +1058,19 @@ def _showPreferencesConsole(script=None, inputEvent=None):
 
     return True
 
+def helpForOrca(script=None, inputEvent=None):
+    """Show Orca Help window (part of the GNOME Access Guide).
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    props = { gnome.PARAM_APP_DATADIR : '/usr/share'}
+    prog = gnome.program_init('orca', '1.0', properties=props)
+    gnome.help_display_with_doc_id(prog, 'gnome-access-guide', 
+                                         'gnome-access-guide', 'ats-2')
+
+    return True
+
 def quitOrca(script=None, inputEvent=None):
     """Quit Orca. Check if the user wants to confirm this action.
     If so, show the confirmation GUI otherwise just shutdown.
